# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""DTITK utility interfaces

DTI-TK developed by Gary Hui Zhang, gary.zhang@ucl.ac.uk
For additional help, visit http://dti-tk.sf.net

The high-dimensional tensor-based DTI registration algorithm

Zhang, H., Avants, B.B, Yushkevich, P.A., Woo, J.H., Wang, S., McCluskey, L.H.,
Elman, L.B., Melhem, E.R., Gee, J.C., High-dimensional spatial normalization of
diffusion tensor images improves the detection of white matter differences in
amyotrophic lateral sclerosis, IEEE Transactions on Medical Imaging,
26(11):1585-1597, November 2007. PMID: 18041273.

The original piecewise-affine tensor-based DTI registration algorithm at the
core of DTI-TK

Zhang, H., Yushkevich, P.A., Alexander, D.C., Gee, J.C., Deformable
registration of diffusion tensor MR images with explicit orientation
optimization, Medical Image Analysis, 10(5):764-785, October 2006. PMID:
16899392.

"""

from ..base import TraitedSpec, CommandLineInputSpec, File, traits, isdefined
from ...utils.filemanip import fname_presuffix
from .base import CommandLineDtitk, DTITKRenameMixin
import os

__docformat__ = "restructuredtext"


class TVAdjustVoxSpInputSpec(CommandLineInputSpec):
    in_file = File(
        desc="tensor volume to modify", exists=True, mandatory=True, argstr="-in %s"
    )
    out_file = File(
        desc="output path",
        argstr="-out %s",
        name_source="in_file",
        name_template="%s_avs",
        keep_extension=True,
    )
    target_file = File(
        desc="target volume to match", argstr="-target %s", xor=["voxel_size", "origin"]
    )
    voxel_size = traits.Tuple(
        (traits.Float(), traits.Float(), traits.Float()),
        desc="xyz voxel size (superseded by target)",
        argstr="-vsize %g %g %g",
        xor=["target_file"],
    )
    origin = traits.Tuple(
        (traits.Float(), traits.Float(), traits.Float()),
        desc="xyz origin (superseded by target)",
        argstr="-origin %g %g %g",
        xor=["target_file"],
    )


class TVAdjustVoxSpOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class TVAdjustVoxSp(CommandLineDtitk):
    """
     Adjusts the voxel space of a tensor volume.

    Example
    -------
    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.TVAdjustVoxSp()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.target_file = 'im2.nii'
    >>> node.cmdline
    'TVAdjustVoxelspace -in im1.nii -out im1_avs.nii -target im2.nii'
    >>> node.run() # doctest: +SKIP

    """

    input_spec = TVAdjustVoxSpInputSpec
    output_spec = TVAdjustVoxSpOutputSpec
    _cmd = "TVAdjustVoxelspace"


class SVAdjustVoxSpInputSpec(CommandLineInputSpec):
    in_file = File(
        desc="scalar volume to modify", exists=True, mandatory=True, argstr="-in %s"
    )
    out_file = File(
        desc="output path",
        argstr="-out %s",
        name_source="in_file",
        name_template="%s_avs",
        keep_extension=True,
    )
    target_file = File(
        desc="target volume to match", argstr="-target %s", xor=["voxel_size", "origin"]
    )
    voxel_size = traits.Tuple(
        (traits.Float(), traits.Float(), traits.Float()),
        desc="xyz voxel size (superseded by target)",
        argstr="-vsize %g %g %g",
        xor=["target_file"],
    )
    origin = traits.Tuple(
        (traits.Float(), traits.Float(), traits.Float()),
        desc="xyz origin (superseded by target)",
        argstr="-origin %g %g %g",
        xor=["target_file"],
    )


class SVAdjustVoxSpOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class SVAdjustVoxSp(CommandLineDtitk):
    """
    Adjusts the voxel space of a scalar volume.

    Example
    -------
    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.SVAdjustVoxSp()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.target_file = 'im2.nii'
    >>> node.cmdline
    'SVAdjustVoxelspace -in im1.nii -out im1_avs.nii -target im2.nii'
    >>> node.run() # doctest: +SKIP

    """

    input_spec = SVAdjustVoxSpInputSpec
    output_spec = SVAdjustVoxSpOutputSpec
    _cmd = "SVAdjustVoxelspace"


class TVResampleInputSpec(CommandLineInputSpec):
    in_file = File(
        desc="tensor volume to resample", exists=True, mandatory=True, argstr="-in %s"
    )
    out_file = File(
        desc="output path",
        name_source="in_file",
        name_template="%s_resampled",
        keep_extension=True,
        argstr="-out %s",
    )
    target_file = File(
        desc="specs read from the target volume",
        argstr="-target %s",
        xor=["array_size", "voxel_size", "origin"],
    )
    align = traits.Enum(
        "center",
        "origin",
        argstr="-align %s",
        desc="how to align output volume to input volume",
    )
    interpolation = traits.Enum(
        "LEI", "EI", argstr="-interp %s", desc="Log Euclidean Euclidean Interpolation"
    )
    array_size = traits.Tuple(
        (traits.Int(), traits.Int(), traits.Int()),
        desc="resampled array size",
        xor=["target_file"],
        argstr="-size %d %d %d",
    )
    voxel_size = traits.Tuple(
        (traits.Float(), traits.Float(), traits.Float()),
        desc="resampled voxel size",
        xor=["target_file"],
        argstr="-vsize %g %g %g",
    )
    origin = traits.Tuple(
        (traits.Float(), traits.Float(), traits.Float()),
        desc="xyz origin",
        xor=["target_file"],
        argstr="-origin %g %g %g",
    )


class TVResampleOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class TVResample(CommandLineDtitk):
    """
    Resamples a tensor volume.

    Example
    -------
    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.TVResample()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.target_file = 'im2.nii'
    >>> node.cmdline
    'TVResample -in im1.nii -out im1_resampled.nii -target im2.nii'
    >>> node.run() # doctest: +SKIP

    """

    input_spec = TVResampleInputSpec
    output_spec = TVResampleOutputSpec
    _cmd = "TVResample"


class SVResampleInputSpec(CommandLineInputSpec):
    in_file = File(
        desc="image to resample", exists=True, mandatory=True, argstr="-in %s"
    )
    out_file = File(
        desc="output path",
        name_source="in_file",
        name_template="%s_resampled",
        keep_extension=True,
        argstr="-out %s",
    )
    target_file = File(
        desc="specs read from the target volume",
        argstr="-target %s",
        xor=["array_size", "voxel_size", "origin"],
    )
    align = traits.Enum(
        "center",
        "origin",
        argstr="-align %s",
        desc="how to align output volume to input volume",
    )
    array_size = traits.Tuple(
        (traits.Int(), traits.Int(), traits.Int()),
        desc="resampled array size",
        xor=["target_file"],
        argstr="-size %d %d %d",
    )
    voxel_size = traits.Tuple(
        (traits.Float(), traits.Float(), traits.Float()),
        desc="resampled voxel size",
        xor=["target_file"],
        argstr="-vsize %g %g %g",
    )
    origin = traits.Tuple(
        (traits.Float(), traits.Float(), traits.Float()),
        desc="xyz origin",
        xor=["target_file"],
        argstr="-origin %g %g %g",
    )


class SVResampleOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class SVResample(CommandLineDtitk):
    """
    Resamples a scalar volume.

    Example
    -------
    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.SVResample()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.target_file = 'im2.nii'
    >>> node.cmdline
    'SVResample -in im1.nii -out im1_resampled.nii -target im2.nii'
    >>> node.run() # doctest: +SKIP

    """

    input_spec = SVResampleInputSpec
    output_spec = SVResampleOutputSpec
    _cmd = "SVResample"


class TVtoolInputSpec(CommandLineInputSpec):
    in_file = File(
        desc="scalar volume to resample", exists=True, argstr="-in %s", mandatory=True
    )
    """NOTE: there are a lot more options here; not implementing all of them"""
    in_flag = traits.Enum("fa", "tr", "ad", "rd", "pd", "rgb", argstr="-%s", desc="")
    out_file = File(argstr="-out %s", genfile=True)


class TVtoolOutputSpec(TraitedSpec):
    out_file = File()


class TVtool(CommandLineDtitk):
    """
    Calculates a tensor metric volume from a tensor volume.

    Example
    -------
    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.TVtool()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.in_flag = 'fa'
    >>> node.cmdline
    'TVtool -in im1.nii -fa -out im1_fa.nii'
    >>> node.run() # doctest: +SKIP

    """

    input_spec = TVtoolInputSpec
    output_spec = TVtoolOutputSpec
    _cmd = "TVtool"

    def _list_outputs(self):
        outputs = self._outputs().get()
        out_file = self.inputs.out_file
        if not isdefined(out_file):
            out_file = self._gen_filename("out_file")
        outputs["out_file"] = os.path.abspath(out_file)
        return outputs

    def _gen_filename(self, name):
        if name != "out_file":
            return
        return fname_presuffix(
            os.path.basename(self.inputs.in_file), suffix="_" + self.inputs.in_flag
        )


"""Note: SVTool not implemented at this time"""


class BinThreshInputSpec(CommandLineInputSpec):
    in_file = File(
        desc="Image to threshold/binarize",
        exists=True,
        position=0,
        argstr="%s",
        mandatory=True,
    )
    out_file = File(
        desc="output path",
        position=1,
        argstr="%s",
        keep_extension=True,
        name_source="in_file",
        name_template="%s_thrbin",
    )
    lower_bound = traits.Float(
        0.01,
        usedefault=True,
        position=2,
        argstr="%g",
        mandatory=True,
        desc="lower bound of binarization range",
    )
    upper_bound = traits.Float(
        100,
        usedefault=True,
        position=3,
        argstr="%g",
        mandatory=True,
        desc="upper bound of binarization range",
    )
    inside_value = traits.Float(
        1,
        position=4,
        argstr="%g",
        usedefault=True,
        mandatory=True,
        desc="value for voxels in " "binarization range",
    )
    outside_value = traits.Float(
        0,
        position=5,
        argstr="%g",
        usedefault=True,
        mandatory=True,
        desc="value for voxels" "outside of binarization range",
    )


class BinThreshOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class BinThresh(CommandLineDtitk):
    """
    Binarizes an image.

    Example
    -------
    >>> from nipype.interfaces import dtitk
    >>> node = dtitk.BinThresh()
    >>> node.inputs.in_file = 'im1.nii'
    >>> node.inputs.lower_bound = 0
    >>> node.inputs.upper_bound = 100
    >>> node.inputs.inside_value = 1
    >>> node.inputs.outside_value = 0
    >>> node.cmdline
    'BinaryThresholdImageFilter im1.nii im1_thrbin.nii 0 100 1 0'
    >>> node.run() # doctest: +SKIP

    """

    input_spec = BinThreshInputSpec
    output_spec = BinThreshOutputSpec
    _cmd = "BinaryThresholdImageFilter"


class BinThreshTask(DTITKRenameMixin, BinThresh):
    pass


class SVAdjustVoxSpTask(DTITKRenameMixin, SVAdjustVoxSp):
    pass


class SVResampleTask(DTITKRenameMixin, SVResample):
    pass


class TVAdjustOriginTask(DTITKRenameMixin, TVAdjustVoxSp):
    pass


class TVAdjustVoxSpTask(DTITKRenameMixin, TVAdjustVoxSp):
    pass


class TVResampleTask(DTITKRenameMixin, TVResample):
    pass


class TVtoolTask(DTITKRenameMixin, TVtool):
    pass



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""PETPVC is a toolbox for partial volume correction in positron emission tomography."""
import os

from .base import (
    TraitedSpec,
    CommandLineInputSpec,
    CommandLine,
    File,
    isdefined,
    traits,
)
from ..utils.filemanip import fname_presuffix
from ..external.due import BibTeX

pvc_methods = [
    "GTM",
    "IY",
    "IY+RL",
    "IY+VC",
    "LABBE",
    "LABBE+MTC",
    "LABBE+MTC+RL",
    "LABBE+MTC+VC",
    "LABBE+RBV",
    "LABBE+RBV+RL",
    "LABBE+RBV+VC",
    "MG",
    "MG+RL",
    "MG+VC",
    "MTC",
    "MTC+RL",
    "MTC+VC",
    "RBV",
    "RBV+RL",
    "RBV+VC",
    "RL",
    "VC",
]


class PETPVCInputSpec(CommandLineInputSpec):
    in_file = File(desc="PET image file", exists=True, mandatory=True, argstr="-i %s")
    out_file = File(desc="Output file", genfile=True, hash_files=False, argstr="-o %s")
    mask_file = File(
        desc="Mask image file", exists=True, mandatory=True, argstr="-m %s"
    )
    pvc = traits.Enum(
        pvc_methods,
        mandatory=True,
        argstr="-p %s",
        desc="""\
Desired PVC method:

    * Geometric transfer matrix -- ``GTM``
    * Labbe approach -- ``LABBE``
    * Richardson-Lucy -- ``RL``
    * Van-Cittert -- ``VC``
    * Region-based voxel-wise correction -- ``RBV``
    * RBV with Labbe -- ``LABBE+RBV``
    * RBV with Van-Cittert -- ``RBV+VC``
    * RBV with Richardson-Lucy -- ``RBV+RL``
    * RBV with Labbe and Van-Cittert -- ``LABBE+RBV+VC``
    * RBV with Labbe and Richardson-Lucy -- ``LABBE+RBV+RL``
    * Multi-target correction -- ``MTC``
    * MTC with Labbe -- ``LABBE+MTC``
    * MTC with Van-Cittert -- ``MTC+VC``
    * MTC with Richardson-Lucy -- ``MTC+RL``
    * MTC with Labbe and Van-Cittert -- ``LABBE+MTC+VC``
    * MTC with Labbe and Richardson-Lucy -- ``LABBE+MTC+RL``
    * Iterative Yang -- ``IY``
    * Iterative Yang with Van-Cittert -- ``IY+VC``
    * Iterative Yang with Richardson-Lucy -- ``IY+RL``
    * Muller Gartner -- ``MG``
    * Muller Gartner with Van-Cittert -- ``MG+VC``
    * Muller Gartner with Richardson-Lucy -- ``MG+RL``

""",
    )
    fwhm_x = traits.Float(
        desc="The full-width at half maximum in mm along x-axis",
        mandatory=True,
        argstr="-x %.4f",
    )
    fwhm_y = traits.Float(
        desc="The full-width at half maximum in mm along y-axis",
        mandatory=True,
        argstr="-y %.4f",
    )
    fwhm_z = traits.Float(
        desc="The full-width at half maximum in mm along z-axis",
        mandatory=True,
        argstr="-z %.4f",
    )
    debug = traits.Bool(
        desc="Prints debug information",
        usedefault=True,
        default_value=False,
        argstr="-d",
    )
    n_iter = traits.Int(
        desc="Number of iterations", default_value=10, usedefault=True, argstr="-n %d"
    )
    n_deconv = traits.Int(
        desc="Number of deconvolution iterations",
        default_value=10,
        usedefault=True,
        argstr="-k %d",
    )
    alpha = traits.Float(
        desc="Alpha value", default_value=1.5, usedefault=True, argstr="-a %.4f"
    )
    stop_crit = traits.Float(
        desc="Stopping criterion", default_value=0.01, usedefault=True, argstr="-s %.4f"
    )


class PETPVCOutputSpec(TraitedSpec):
    out_file = File(desc="Output file")


class PETPVC(CommandLine):
    """Use PETPVC for partial volume correction of PET images.

    PETPVC ([1]_, [2]_) is a software from the Nuclear Medicine Department
    of the UCL University Hospital, London, UK.

    Examples
    --------
    >>> from ..testing import example_data
    >>> #TODO get data for PETPVC
    >>> pvc = PETPVC()
    >>> pvc.inputs.in_file   = 'pet.nii.gz'
    >>> pvc.inputs.mask_file = 'tissues.nii.gz'
    >>> pvc.inputs.out_file  = 'pet_pvc_rbv.nii.gz'
    >>> pvc.inputs.pvc = 'RBV'
    >>> pvc.inputs.fwhm_x = 2.0
    >>> pvc.inputs.fwhm_y = 2.0
    >>> pvc.inputs.fwhm_z = 2.0
    >>> outs = pvc.run() #doctest: +SKIP

    References
    ----------
    .. [1] K. Erlandsson, I. Buvat, P. H. Pretorius, B. A. Thomas, and B. F. Hutton,
           "A review of partial volume correction techniques for emission tomography
           and their applications in neurology, cardiology and oncology," Phys. Med.
           Biol., vol. 57, no. 21, p. R119, 2012.
    .. [2] https://github.com/UCL/PETPVC

    """

    input_spec = PETPVCInputSpec
    output_spec = PETPVCOutputSpec
    _cmd = "petpvc"

    _references = [
        {
            "entry": BibTeX(
                "@article{0031-9155-61-22-7975,"
                "author={Benjamin A Thomas and Vesna Cuplov and Alexandre Bousse and "
                "Adriana Mendes and Kris Thielemans and Brian F Hutton and Kjell Erlandsson},"
                "title={PETPVC: a toolbox for performing partial volume correction "
                "techniques in positron emission tomography},"
                "journal={Physics in Medicine and Biology},"
                "volume={61},"
                "number={22},"
                "pages={7975},"
                "url={http://stacks.iop.org/0031-9155/61/i=22/a=7975},"
                "doi={https://doi.org/10.1088/0031-9155/61/22/7975},"
                "year={2016},"
                "}"
            ),
            "description": "PETPVC software implementation publication",
            "tags": ["implementation"],
        }
    ]

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs["out_file"] = self.inputs.out_file
        if not isdefined(outputs["out_file"]):
            method_name = self.inputs.pvc.lower()
            outputs["out_file"] = self._gen_fname(
                self.inputs.in_file, suffix="_{}_pvc".format(method_name)
            )

        outputs["out_file"] = os.path.abspath(outputs["out_file"])
        return outputs

    def _gen_fname(
        self, basename, cwd=None, suffix=None, change_ext=True, ext=".nii.gz"
    ):
        """Generate a filename based on the given parameters.

        The filename will take the form: cwd/basename<suffix><ext>.
        If change_ext is True, it will use the extentions specified in
        <instance>intputs.output_type.

        Parameters
        ----------
        basename : str
            Filename to base the new filename on.
        cwd : str
            Path to prefix to the new filename. (default is os.getcwd())
        suffix : str
            Suffix to add to the `basename`.  (defaults is '' )
        change_ext : bool
            Flag to change the filename extension to the given `ext`.
            (Default is False)

        Returns
        -------
        fname : str
            New filename based on given parameters.

        """
        if basename == "":
            msg = "Unable to generate filename for command %s. " % self.cmd
            msg += "basename is not set!"
            raise ValueError(msg)
        if cwd is None:
            cwd = os.getcwd()
        if change_ext:
            if suffix:
                suffix = "".join((suffix, ext))
            else:
                suffix = ext
        if suffix is None:
            suffix = ""
        fname = fname_presuffix(basename, suffix=suffix, use_ext=False, newpath=cwd)
        return fname

    def _gen_filename(self, name):
        if name == "out_file":
            return self._list_outputs()["out_file"]
        return None



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..epi import EddyQuad


def test_EddyQuad_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        base_name=dict(
            argstr="%s",
            position=0,
            usedefault=True,
        ),
        bval_file=dict(
            argstr="--bvals %s",
            extensions=None,
            mandatory=True,
        ),
        bvec_file=dict(
            argstr="--bvecs %s",
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        field=dict(
            argstr="--field %s",
            extensions=None,
        ),
        idx_file=dict(
            argstr="--eddyIdx %s",
            extensions=None,
            mandatory=True,
        ),
        mask_file=dict(
            argstr="--mask %s",
            extensions=None,
            mandatory=True,
        ),
        output_dir=dict(
            argstr="--output-dir %s",
            name_source=["base_name"],
            name_template="%s.qc",
        ),
        output_type=dict(),
        param_file=dict(
            argstr="--eddyParams %s",
            extensions=None,
            mandatory=True,
        ),
        slice_spec=dict(
            argstr="--slspec %s",
            extensions=None,
        ),
        verbose=dict(
            argstr="--verbose",
        ),
    )
    inputs = EddyQuad.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_EddyQuad_outputs():
    output_map = dict(
        avg_b0_pe_png=dict(),
        avg_b_png=dict(),
        clean_volumes=dict(
            extensions=None,
        ),
        cnr_png=dict(),
        qc_json=dict(
            extensions=None,
        ),
        qc_pdf=dict(
            extensions=None,
        ),
        residuals=dict(
            extensions=None,
        ),
        vdm_png=dict(
            extensions=None,
        ),
    )
    outputs = EddyQuad.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..confounds import TCompCor


def test_TCompCor_inputs():
    input_map = dict(
        components_file=dict(
            usedefault=True,
        ),
        failure_mode=dict(
            usedefault=True,
        ),
        header_prefix=dict(),
        high_pass_cutoff=dict(
            usedefault=True,
        ),
        ignore_initial_volumes=dict(
            usedefault=True,
        ),
        mask_files=dict(),
        mask_index=dict(
            requires=["mask_files"],
            xor=["merge_method"],
        ),
        mask_names=dict(),
        merge_method=dict(
            requires=["mask_files"],
            xor=["mask_index"],
        ),
        num_components=dict(
            xor=["variance_threshold"],
        ),
        percentile_threshold=dict(
            usedefault=True,
        ),
        pre_filter=dict(
            usedefault=True,
        ),
        realigned_file=dict(
            extensions=None,
            mandatory=True,
        ),
        regress_poly_degree=dict(
            usedefault=True,
        ),
        repetition_time=dict(),
        save_metadata=dict(
            usedefault=True,
        ),
        save_pre_filter=dict(
            usedefault=True,
        ),
        use_regress_poly=dict(
            deprecated="0.15.0",
            new_name="pre_filter",
        ),
        variance_threshold=dict(
            xor=["num_components"],
        ),
    )
    inputs = TCompCor.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_TCompCor_outputs():
    output_map = dict(
        components_file=dict(
            extensions=None,
        ),
        high_variance_masks=dict(),
        metadata_file=dict(
            extensions=None,
        ),
        pre_filter_file=dict(
            extensions=None,
        ),
    )
    outputs = TCompCor.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Interfaces to perform image registrations and to apply the resulting
displacement maps to images and points.

"""
import os.path as op
import re

from ... import logging
from .base import ElastixBaseInputSpec
from ..base import CommandLine, TraitedSpec, File, traits, InputMultiPath

iflogger = logging.getLogger("nipype.interface")


class RegistrationInputSpec(ElastixBaseInputSpec):
    fixed_image = File(exists=True, mandatory=True, argstr="-f %s", desc="fixed image")
    moving_image = File(
        exists=True, mandatory=True, argstr="-m %s", desc="moving image"
    )
    parameters = InputMultiPath(
        File(exists=True),
        mandatory=True,
        argstr="-p %s...",
        desc="parameter file, elastix handles 1 or more -p",
    )
    fixed_mask = File(exists=True, argstr="-fMask %s", desc="mask for fixed image")
    moving_mask = File(exists=True, argstr="-mMask %s", desc="mask for moving image")
    initial_transform = File(
        exists=True, argstr="-t0 %s", desc="parameter file for initial transform"
    )


class RegistrationOutputSpec(TraitedSpec):
    transform = InputMultiPath(File(exists=True), desc="output transform")
    warped_file = File(desc="input moving image warped to fixed image")
    warped_files = InputMultiPath(
        File(exists=False),
        desc=("input moving image warped to fixed image at each level"),
    )
    warped_files_flags = traits.List(
        traits.Bool(False), desc="flag indicating if warped image was generated"
    )


class Registration(CommandLine):
    """
    Elastix nonlinear registration interface

    Example
    -------

    >>> from nipype.interfaces.elastix import Registration
    >>> reg = Registration()
    >>> reg.inputs.fixed_image = 'fixed1.nii'
    >>> reg.inputs.moving_image = 'moving1.nii'
    >>> reg.inputs.parameters = ['elastix.txt']
    >>> reg.cmdline
    'elastix -f fixed1.nii -m moving1.nii -threads 1 -out ./ -p elastix.txt'


    """

    _cmd = "elastix"
    input_spec = RegistrationInputSpec
    output_spec = RegistrationOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()

        out_dir = op.abspath(self.inputs.output_path)

        regex = re.compile(r"^\((\w+)\s(.+)\)$")

        outputs["transform"] = []
        outputs["warped_files"] = []
        outputs["warped_files_flags"] = []

        for i, params in enumerate(self.inputs.parameters):
            config = {}

            with open(params, "r") as f:
                for line in f.readlines():
                    line = line.strip()
                    if not line.startswith("//") and line:
                        m = regex.search(line)
                        if m:
                            value = self._cast(m.group(2).strip())
                            config[m.group(1).strip()] = value

            outputs["transform"].append(
                op.join(out_dir, "TransformParameters.%01d.txt" % i)
            )

            warped_file = None
            if config["WriteResultImage"]:
                warped_file = op.join(
                    out_dir, "result.%01d.%s" % (i, config["ResultImageFormat"])
                )

            outputs["warped_files"].append(warped_file)
            outputs["warped_files_flags"].append(config["WriteResultImage"])

        if outputs["warped_files_flags"][-1]:
            outputs["warped_file"] = outputs["warped_files"][-1]

        return outputs

    def _cast(self, val):
        if val.startswith('"') and val.endswith('"'):
            if val == '"true"':
                return True
            elif val == '"false"':
                return False
            else:
                return val[1:-1]

        try:
            return int(val)
        except ValueError:
            try:
                return float(val)
            except ValueError:
                return val


class ApplyWarpInputSpec(ElastixBaseInputSpec):
    transform_file = File(
        exists=True,
        mandatory=True,
        argstr="-tp %s",
        desc="transform-parameter file, only 1",
    )

    moving_image = File(
        exists=True, argstr="-in %s", mandatory=True, desc="input image to deform"
    )


class ApplyWarpOutputSpec(TraitedSpec):
    warped_file = File(desc="input moving image warped to fixed image")


class ApplyWarp(CommandLine):
    """
    Use ``transformix`` to apply a transform on an input image.
    The transform is specified in the transform-parameter file.

    Example
    -------

    >>> from nipype.interfaces.elastix import ApplyWarp
    >>> reg = ApplyWarp()
    >>> reg.inputs.moving_image = 'moving1.nii'
    >>> reg.inputs.transform_file = 'TransformParameters.0.txt'
    >>> reg.cmdline
    'transformix -in moving1.nii -threads 1 -out ./ -tp TransformParameters.0.txt'


    """

    _cmd = "transformix"
    input_spec = ApplyWarpInputSpec
    output_spec = ApplyWarpOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        out_dir = op.abspath(self.inputs.output_path)
        outputs["warped_file"] = op.join(out_dir, "result.nii.gz")
        return outputs


class AnalyzeWarpInputSpec(ApplyWarpInputSpec):
    points = traits.Enum(
        "all",
        usedefault=True,
        position=0,
        argstr="-def %s",
        desc="transform all points from the input-image, which effectively"
        " generates a deformation field.",
    )
    jac = traits.Enum(
        "all",
        usedefault=True,
        argstr="-jac %s",
        desc="generate an image with the determinant of the spatial Jacobian",
    )
    jacmat = traits.Enum(
        "all",
        usedefault=True,
        argstr="-jacmat %s",
        desc="generate an image with the spatial Jacobian matrix at each voxel",
    )
    moving_image = File(
        exists=True, argstr="-in %s", desc="input image to deform (not used)"
    )


class AnalyzeWarpOutputSpec(TraitedSpec):
    disp_field = File(desc="displacements field")
    jacdet_map = File(desc="det(Jacobian) map")
    jacmat_map = File(desc="Jacobian matrix map")


class AnalyzeWarp(ApplyWarp):
    """
    Use transformix to get details from the input transform (generate
    the corresponding deformation field, generate the determinant of the
    Jacobian map or the Jacobian map itself)

    Example
    -------

    >>> from nipype.interfaces.elastix import AnalyzeWarp
    >>> reg = AnalyzeWarp()
    >>> reg.inputs.transform_file = 'TransformParameters.0.txt'
    >>> reg.cmdline
    'transformix -def all -jac all -jacmat all -threads 1 -out ./ -tp TransformParameters.0.txt'


    """

    input_spec = AnalyzeWarpInputSpec
    output_spec = AnalyzeWarpOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        out_dir = op.abspath(self.inputs.output_path)
        outputs["disp_field"] = op.join(out_dir, "deformationField.nii.gz")
        outputs["jacdet_map"] = op.join(out_dir, "spatialJacobian.nii.gz")
        outputs["jacmat_map"] = op.join(out_dir, "fullSpatialJacobian.nii.gz")
        return outputs


class PointsWarpInputSpec(ElastixBaseInputSpec):
    points_file = File(
        exists=True,
        argstr="-def %s",
        mandatory=True,
        desc="input points (accepts .vtk triangular meshes).",
    )
    transform_file = File(
        exists=True,
        mandatory=True,
        argstr="-tp %s",
        desc="transform-parameter file, only 1",
    )


class PointsWarpOutputSpec(TraitedSpec):
    warped_file = File(desc="input points displaced in fixed image domain")


class PointsWarp(CommandLine):
    """Use ``transformix`` to apply a transform on an input point set.
    The transform is specified in the transform-parameter file.

    Example
    -------

    >>> from nipype.interfaces.elastix import PointsWarp
    >>> reg = PointsWarp()
    >>> reg.inputs.points_file = 'surf1.vtk'
    >>> reg.inputs.transform_file = 'TransformParameters.0.txt'
    >>> reg.cmdline
    'transformix -threads 1 -out ./ -def surf1.vtk -tp TransformParameters.0.txt'


    """

    _cmd = "transformix"
    input_spec = PointsWarpInputSpec
    output_spec = PointsWarpOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        out_dir = op.abspath(self.inputs.output_path)

        fname, ext = op.splitext(op.basename(self.inputs.points_file))

        outputs["warped_file"] = op.join(out_dir, "outputpoints%s" % ext)
        return outputs



# -*- coding: utf-8 -*-
"""Parallel workflow execution via PBS/Torque
"""
import os
from time import sleep

from ... import logging
from ...interfaces.base import CommandLine
from .base import SGELikeBatchManagerBase, logger

iflogger = logging.getLogger("nipype.interface")


class PBSPlugin(SGELikeBatchManagerBase):
    """Execute using PBS/Torque

    The plugin_args input to run can be used to control the SGE execution.
    Currently supported options are:

    - template : template to use for batch job submission
    - qsub_args : arguments to be prepended to the job execution script in the
                  qsub call
    - max_jobname_len: maximum length of the job name.  Default 15.

    """

    # Addtional class variables
    _max_jobname_len = 15

    def __init__(self, **kwargs):
        template = """
#PBS -V
        """
        self._retry_timeout = 2
        self._max_tries = 2
        self._max_jobname_length = 15
        if "plugin_args" in kwargs and kwargs["plugin_args"]:
            if "retry_timeout" in kwargs["plugin_args"]:
                self._retry_timeout = kwargs["plugin_args"]["retry_timeout"]
            if "max_tries" in kwargs["plugin_args"]:
                self._max_tries = kwargs["plugin_args"]["max_tries"]
            if "max_jobname_len" in kwargs["plugin_args"]:
                self._max_jobname_len = kwargs["plugin_args"]["max_jobname_len"]
        super(PBSPlugin, self).__init__(template, **kwargs)

    def _is_pending(self, taskid):
        result = CommandLine(
            "qstat -f {}".format(taskid),
            environ=dict(os.environ),
            terminal_output="file_split",
            resource_monitor=False,
            ignore_exception=True,
        ).run()

        stdout = result.runtime.stdout
        stderr = result.runtime.stderr
        errmsg = "Unknown Job Id"
        success = "Job has finished"
        if (success in stderr) or ("job_state = C" in stdout):
            return False
        else:
            return errmsg not in stderr

    def _submit_batchtask(self, scriptfile, node):
        cmd = CommandLine(
            "qsub",
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output="allatonce",
        )
        path = os.path.dirname(scriptfile)
        qsubargs = ""
        if self._qsub_args:
            qsubargs = self._qsub_args
        if "qsub_args" in node.plugin_args:
            if "overwrite" in node.plugin_args and node.plugin_args["overwrite"]:
                qsubargs = node.plugin_args["qsub_args"]
            else:
                qsubargs += " " + node.plugin_args["qsub_args"]
        if "-o" not in qsubargs:
            qsubargs = "%s -o %s" % (qsubargs, path)
        if "-e" not in qsubargs:
            qsubargs = "%s -e %s" % (qsubargs, path)
        if node._hierarchy:
            jobname = ".".join((dict(os.environ)["LOGNAME"], node._hierarchy, node._id))
        else:
            jobname = ".".join((dict(os.environ)["LOGNAME"], node._id))
        jobnameitems = jobname.split(".")
        jobnameitems.reverse()
        jobname = ".".join(jobnameitems)
        jobname = jobname[0 : self._max_jobname_len]
        cmd.inputs.args = "%s -N %s %s" % (qsubargs, jobname, scriptfile)

        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName("CRITICAL"))
        tries = 0
        while True:
            try:
                result = cmd.run()
            except Exception as e:
                if tries < self._max_tries:
                    tries += 1
                    # sleep 2 seconds and try again.
                    sleep(self._retry_timeout)
                else:
                    iflogger.setLevel(oldlevel)
                    raise RuntimeError(
                        "Could not submit pbs task for node {}\n{}".format(node._id, e)
                    )
            else:
                break
        iflogger.setLevel(oldlevel)
        # retrieve pbs taskid
        taskid = result.runtime.stdout.split(".")[0]
        self._pending[taskid] = node.output_dir()
        logger.debug("submitted pbs task: {} for node {}".format(taskid, node._id))

        return taskid



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import DARTEL


def test_DARTEL_inputs():
    input_map = dict(
        image_files=dict(
            copyfile=False,
            field="warp.images",
            mandatory=True,
        ),
        iteration_parameters=dict(
            field="warp.settings.param",
        ),
        matlab_cmd=dict(),
        mfile=dict(
            usedefault=True,
        ),
        optimization_parameters=dict(
            field="warp.settings.optim",
        ),
        paths=dict(),
        regularization_form=dict(
            field="warp.settings.rform",
        ),
        template_prefix=dict(
            field="warp.settings.template",
            usedefault=True,
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver="8",
            usedefault=True,
        ),
    )
    inputs = DARTEL.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_DARTEL_outputs():
    output_map = dict(
        dartel_flow_fields=dict(),
        final_template_file=dict(
            extensions=None,
        ),
        template_files=dict(),
    )
    outputs = DARTEL.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from .morphology import GrayscaleGrindPeakImageFilter, GrayscaleFillHoleImageFilter
from .denoising import (
    GradientAnisotropicDiffusion,
    CurvatureAnisotropicDiffusion,
    GaussianBlurImageFilter,
    MedianImageFilter,
)
from .arithmetic import (
    MultiplyScalarVolumes,
    MaskScalarVolume,
    SubtractScalarVolumes,
    AddScalarVolumes,
    CastScalarVolume,
)
from .extractskeleton import ExtractSkeleton
from .histogrammatching import HistogramMatching
from .thresholdscalarvolume import ThresholdScalarVolume
from .n4itkbiasfieldcorrection import N4ITKBiasFieldCorrection
from .checkerboardfilter import CheckerBoardFilter
from .imagelabelcombine import ImageLabelCombine
from .votingbinaryholefillingimagefilter import VotingBinaryHoleFillingImageFilter
from .resamplescalarvectordwivolume import ResampleScalarVectorDWIVolume



# -*- coding: utf-8 -*-

if __name__ == "__main__":
    from nipype.interfaces.slicer.generate_classes import generate_all_classes

    # NOTE:  For now either the launcher needs to be found on the default path, or
    # every tool in the modules list must be found on the default path
    # AND calling the module with --xml must be supported and compliant.
    modules_list = [
        "edu.jhu.bme.smile.demo.RandomVol",
        "de.mpg.cbs.jist.laminar.JistLaminarProfileCalculator",
        "de.mpg.cbs.jist.laminar.JistLaminarProfileSampling",
        "de.mpg.cbs.jist.laminar.JistLaminarROIAveraging",
        "de.mpg.cbs.jist.laminar.JistLaminarVolumetricLayering",
        "de.mpg.cbs.jist.laminar.JistLaminarProfileGeometry",
        "de.mpg.cbs.jist.brain.JistBrainMgdmSegmentation",
        "de.mpg.cbs.jist.brain.JistBrainMp2rageSkullStripping",
        "de.mpg.cbs.jist.brain.JistBrainPartialVolumeFilter",
        "de.mpg.cbs.jist.brain.JistBrainMp2rageDuraEstimation",
    ]

    modules_from_chris = [
        "edu.jhu.ece.iacl.plugins.segmentation.skull_strip.MedicAlgorithmSPECTRE2010",
        "edu.jhu.ece.iacl.plugins.utilities.volume.MedicAlgorithmMipavReorient",
        "edu.jhu.ece.iacl.plugins.utilities.math.MedicAlgorithmImageCalculator",
        "de.mpg.cbs.jist.brain.JistBrainMp2rageDuraEstimation",
        "de.mpg.cbs.jist.brain.JistBrainPartialVolumeFilter",
        "edu.jhu.ece.iacl.plugins.utilities.volume.MedicAlgorithmThresholdToBinaryMask",
        # 'de.mpg.cbs.jist.cortex.JistCortexFullCRUISE', # waiting for http://www.nitrc.org/tracker/index.php?func=detail&aid=7236&group_id=228&atid=942 to be fixed
        "de.mpg.cbs.jist.cortex.JistCortexSurfaceMeshInflation",
    ]

    modules_from_julia = [
        "de.mpg.cbs.jist.intensity.JistIntensityMp2rageMasking",
        "edu.jhu.ece.iacl.plugins.segmentation.skull_strip.MedicAlgorithmSPECTRE2010",
    ]

    modules_from_leonie = [
        "edu.jhu.ece.iacl.plugins.classification.MedicAlgorithmLesionToads"
    ]

    modules_from_yasinyazici = [
        "edu.jhu.ece.iacl.plugins.classification.MedicAlgorithmN3"
    ]

    modules_list = list(
        set(modules_list)
        .union(modules_from_chris)
        .union(modules_from_leonie)
        .union(modules_from_julia)
        .union(modules_from_yasinyazici)
        .union(modules_list)
    )

    generate_all_classes(
        modules_list=modules_list,
        launcher=["java edu.jhu.ece.iacl.jist.cli.run"],
        redirect_x=True,
        mipav_hacks=True,
    )



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..specialized import FiducialRegistration


def test_FiducialRegistration_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixedLandmarks=dict(
            argstr="--fixedLandmarks %s...",
        ),
        movingLandmarks=dict(
            argstr="--movingLandmarks %s...",
        ),
        outputMessage=dict(
            argstr="--outputMessage %s",
        ),
        rms=dict(
            argstr="--rms %f",
        ),
        saveTransform=dict(
            argstr="--saveTransform %s",
            hash_files=False,
        ),
        transformType=dict(
            argstr="--transformType %s",
        ),
    )
    inputs = FiducialRegistration.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FiducialRegistration_outputs():
    output_map = dict(
        saveTransform=dict(
            extensions=None,
        ),
    )
    outputs = FiducialRegistration.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import MRIsExpand


def test_MRIsExpand_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        distance=dict(
            argstr="%g",
            mandatory=True,
            position=-2,
        ),
        dt=dict(
            argstr="-T %g",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        nsurfaces=dict(
            argstr="-N %d",
        ),
        out_name=dict(
            argstr="%s",
            position=-1,
            usedefault=True,
        ),
        pial=dict(
            argstr="-pial %s",
            copyfile=False,
        ),
        smooth_averages=dict(
            argstr="-A %d",
        ),
        sphere=dict(
            copyfile=False,
            usedefault=True,
        ),
        spring=dict(
            argstr="-S %g",
        ),
        subjects_dir=dict(),
        thickness=dict(
            argstr="-thickness",
        ),
        thickness_name=dict(
            argstr="-thickness_name %s",
            copyfile=False,
        ),
        write_iterations=dict(
            argstr="-W %d",
        ),
    )
    inputs = MRIsExpand.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MRIsExpand_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = MRIsExpand.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Nipype interface for seg_maths.

The maths module provides higher-level interfaces to some of the operations
that can be performed with the niftysegmaths (seg_maths) command-line program.

"""

import os

from ..base import (
    TraitedSpec,
    File,
    traits,
    isdefined,
    CommandLineInputSpec,
    NipypeInterfaceError,
)
from .base import NiftySegCommand
from ..niftyreg.base import get_custom_path
from ...utils.filemanip import split_filename


class MathsInput(CommandLineInputSpec):
    """Input Spec for seg_maths interfaces."""

    in_file = File(
        position=2, argstr="%s", exists=True, mandatory=True, desc="image to operate on"
    )

    out_file = File(
        name_source=["in_file"],
        name_template="%s",
        position=-2,
        argstr="%s",
        desc="image to write",
    )

    desc = "datatype to use for output (default uses input type)"
    output_datatype = traits.Enum(
        "float",
        "char",
        "int",
        "short",
        "double",
        "input",
        position=-3,
        argstr="-odt %s",
        desc=desc,
    )


class MathsOutput(TraitedSpec):
    """Output Spec for seg_maths interfaces."""

    out_file = File(desc="image written after calculations")


class MathsCommand(NiftySegCommand):
    """
    Base Command Interface for seg_maths interfaces.

    The executable seg_maths enables the sequential execution of arithmetic
    operations, like multiplication (-mul), division (-div) or addition
    (-add), binarisation (-bin) or thresholding (-thr) operations and
    convolution by a Gaussian kernel (-smo). It also alows mathematical
    morphology based operations like dilation (-dil), erosion (-ero),
    connected components (-lconcomp) and hole filling (-fill), Euclidean
    (- euc) and geodesic (-geo) distance transforms, local image similarity
    metric calculation (-lncc and -lssd). Finally, it allows multiple
    operations over the dimensionality of the image, from merging 3D images
    together as a 4D image (-merge) or splitting (-split or -tp) 4D images
    into several 3D images, to estimating the maximum, minimum and average
    over all time-points, etc.
    """

    _cmd = get_custom_path("seg_maths", env_dir="NIFTYSEGDIR")
    input_spec = MathsInput
    output_spec = MathsOutput
    _suffix = "_maths"

    def _overload_extension(self, value, name=None):
        path, base, _ = split_filename(value)
        _, _, ext = split_filename(self.inputs.in_file)

        suffix = self._suffix
        if suffix != "_merged" and isdefined(self.inputs.operation):
            suffix = "_" + self.inputs.operation

        return os.path.join(path, "{0}{1}{2}".format(base, suffix, ext))


class UnaryMathsInput(MathsInput):
    """Input Spec for seg_maths Unary operations."""

    operation = traits.Enum(
        "sqrt",
        "exp",
        "log",
        "recip",
        "abs",
        "bin",
        "otsu",
        "lconcomp",
        "concomp6",
        "concomp26",
        "fill",
        "euc",
        "tpmax",
        "tmean",
        "tmax",
        "tmin",
        "splitlab",
        "removenan",
        "isnan",
        "subsamp2",
        "scl",
        "4to5",
        "range",
        argstr="-%s",
        position=4,
        mandatory=True,
        desc="""\
Operation to perform:

    * sqrt - Square root of the image).
    * exp - Exponential root of the image.
    * log - Log of the image.
    * recip - Reciprocal (1/I) of the image.
    * abs - Absolute value of the image.
    * bin - Binarise the image.
    * otsu - Otsu thresholding of the current image.
    * lconcomp - Take the largest connected component
    * concomp6 - Label the different connected components with a 6NN kernel
    * concomp26 - Label the different connected components with a 26NN kernel
    * fill - Fill holes in binary object (e.g. fill ventricle in brain mask).
    * euc - Euclidean distance transform
    * tpmax - Get the time point with the highest value (binarise 4D probabilities)
    * tmean - Mean value of all time points.
    * tmax - Max value of all time points.
    * tmin - Mean value of all time points.
    * splitlab - Split the integer labels into multiple timepoints
    * removenan - Remove all NaNs and replace then with 0
    * isnan - Binary image equal to 1 if the value is NaN and 0 otherwise
    * subsamp2 - Subsample the image by 2 using NN sampling (qform and sform scaled)
    * scl  - Reset scale and slope info.
    * 4to5 - Flip the 4th and 5th dimension.
    * range - Reset the image range to the min max.

""",
    )


class UnaryMaths(MathsCommand):
    """Unary mathematical operations.

    See Also
    --------
    `Source code <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg>`__ --
    `Documentation <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg_documentation>`__

    Examples
    --------
    >>> import copy
    >>> from nipype.interfaces import niftyseg
    >>> unary = niftyseg.UnaryMaths()
    >>> unary.inputs.output_datatype = 'float'
    >>> unary.inputs.in_file = 'im1.nii'

    >>> # Test sqrt operation
    >>> unary_sqrt = copy.deepcopy(unary)
    >>> unary_sqrt.inputs.operation = 'sqrt'
    >>> unary_sqrt.cmdline
    'seg_maths im1.nii -sqrt -odt float im1_sqrt.nii'
    >>> unary_sqrt.run()  # doctest: +SKIP

    >>> # Test sqrt operation
    >>> unary_abs = copy.deepcopy(unary)
    >>> unary_abs.inputs.operation = 'abs'
    >>> unary_abs.cmdline
    'seg_maths im1.nii -abs -odt float im1_abs.nii'
    >>> unary_abs.run()  # doctest: +SKIP

    >>> # Test bin operation
    >>> unary_bin = copy.deepcopy(unary)
    >>> unary_bin.inputs.operation = 'bin'
    >>> unary_bin.cmdline
    'seg_maths im1.nii -bin -odt float im1_bin.nii'
    >>> unary_bin.run()  # doctest: +SKIP

    >>> # Test otsu operation
    >>> unary_otsu = copy.deepcopy(unary)
    >>> unary_otsu.inputs.operation = 'otsu'
    >>> unary_otsu.cmdline
    'seg_maths im1.nii -otsu -odt float im1_otsu.nii'
    >>> unary_otsu.run()  # doctest: +SKIP

    >>> # Test isnan operation
    >>> unary_isnan = copy.deepcopy(unary)
    >>> unary_isnan.inputs.operation = 'isnan'
    >>> unary_isnan.cmdline
    'seg_maths im1.nii -isnan -odt float im1_isnan.nii'
    >>> unary_isnan.run()  # doctest: +SKIP

    """

    input_spec = UnaryMathsInput


class BinaryMathsInput(MathsInput):
    """Input Spec for seg_maths Binary operations."""

    operation = traits.Enum(
        "mul",
        "div",
        "add",
        "sub",
        "pow",
        "thr",
        "uthr",
        "smo",
        "edge",
        "sobel3",
        "sobel5",
        "min",
        "smol",
        "geo",
        "llsnorm",
        "masknan",
        "hdr_copy",
        "splitinter",
        mandatory=True,
        argstr="-%s",
        position=4,
        desc="""\
Operation to perform:

    * mul - <float/file> - Multiply image <float> value or by other image.
    * div - <float/file> - Divide image by <float> or by other image.
    * add - <float/file> - Add image by <float> or by other image.
    * sub - <float/file> - Subtract image by <float> or by other image.
    * pow - <float> - Image to the power of <float>.
    * thr - <float> - Threshold the image below <float>.
    * uthr - <float> - Threshold image above <float>.
    * smo - <float> - Gaussian smoothing by std <float> (in voxels and up to 4-D).
    * edge - <float> - Calculate the edges of the image using a threshold <float>.
    * sobel3 - <float> - Calculate the edges of all timepoints using a Sobel filter
      with a 3x3x3 kernel and applying <float> gaussian smoothing.
    * sobel5 - <float> - Calculate the edges of all timepoints using a Sobel filter
      with a 5x5x5 kernel and applying <float> gaussian smoothing.
    * min - <file> - Get the min per voxel between <current> and <file>.
    * smol - <float> - Gaussian smoothing of a 3D label image.
    * geo - <float/file> - Geodesic distance according to the speed function <float/file>
    * llsnorm  <file_norm> - Linear LS normalisation between current and <file_norm>
    * masknan <file_norm> - Assign everything outside the mask (mask==0) with NaNs
    * hdr_copy <file> - Copy header from working image to <file> and save in <output>.
    * splitinter <x/y/z> - Split interleaved slices in direction <x/y/z>
      into separate time points

""",
    )

    operand_file = File(
        exists=True,
        argstr="%s",
        mandatory=True,
        position=5,
        xor=["operand_value", "operand_str"],
        desc="second image to perform operation with",
    )

    operand_value = traits.Float(
        argstr="%.8f",
        mandatory=True,
        position=5,
        xor=["operand_file", "operand_str"],
        desc="float value to perform operation with",
    )

    desc = "string value to perform operation splitinter"
    operand_str = traits.Enum(
        "x",
        "y",
        "z",
        argstr="%s",
        mandatory=True,
        position=5,
        xor=["operand_value", "operand_file"],
        desc=desc,
    )


class BinaryMaths(MathsCommand):
    """Binary mathematical operations.

    See Also
    --------
    `Source code <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg>`__ --
    `Documentation <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg_documentation>`__

    Examples
    --------
    >>> import copy
    >>> from nipype.interfaces import niftyseg
    >>> binary = niftyseg.BinaryMaths()
    >>> binary.inputs.in_file = 'im1.nii'
    >>> binary.inputs.output_datatype = 'float'

    >>> # Test sub operation
    >>> binary_sub = copy.deepcopy(binary)
    >>> binary_sub.inputs.operation = 'sub'
    >>> binary_sub.inputs.operand_file = 'im2.nii'
    >>> binary_sub.cmdline
    'seg_maths im1.nii -sub im2.nii -odt float im1_sub.nii'
    >>> binary_sub.run()  # doctest: +SKIP

    >>> # Test mul operation
    >>> binary_mul = copy.deepcopy(binary)
    >>> binary_mul.inputs.operation = 'mul'
    >>> binary_mul.inputs.operand_value = 2.0
    >>> binary_mul.cmdline
    'seg_maths im1.nii -mul 2.00000000 -odt float im1_mul.nii'
    >>> binary_mul.run()  # doctest: +SKIP

    >>> # Test llsnorm operation
    >>> binary_llsnorm = copy.deepcopy(binary)
    >>> binary_llsnorm.inputs.operation = 'llsnorm'
    >>> binary_llsnorm.inputs.operand_file = 'im2.nii'
    >>> binary_llsnorm.cmdline
    'seg_maths im1.nii -llsnorm im2.nii -odt float im1_llsnorm.nii'
    >>> binary_llsnorm.run()  # doctest: +SKIP

    >>> # Test splitinter operation
    >>> binary_splitinter = copy.deepcopy(binary)
    >>> binary_splitinter.inputs.operation = 'splitinter'
    >>> binary_splitinter.inputs.operand_str = 'z'
    >>> binary_splitinter.cmdline
    'seg_maths im1.nii -splitinter z -odt float im1_splitinter.nii'
    >>> binary_splitinter.run()  # doctest: +SKIP

    """

    input_spec = BinaryMathsInput

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for seg_maths."""
        if opt == "operand_str" and self.inputs.operation != "splitinter":
            err = 'operand_str set but with an operation different than \
"splitinter"'

            raise NipypeInterfaceError(err)

        if opt == "operation":
            # Only float
            if val in ["pow", "thr", "uthr", "smo", "edge", "sobel3", "sobel5", "smol"]:
                if not isdefined(self.inputs.operand_value):
                    err = "operand_value not set for {0}.".format(val)
                    raise NipypeInterfaceError(err)
            # only files
            elif val in ["min", "llsnorm", "masknan", "hdr_copy"]:
                if not isdefined(self.inputs.operand_file):
                    err = "operand_file not set for {0}.".format(val)
                    raise NipypeInterfaceError(err)
            # splitinter:
            elif val == "splitinter":
                if not isdefined(self.inputs.operand_str):
                    err = "operand_str not set for splitinter."
                    raise NipypeInterfaceError(err)

        if opt == "operand_value" and float(val) == 0.0:
            return "0"

        return super(BinaryMaths, self)._format_arg(opt, spec, val)

    def _overload_extension(self, value, name=None):
        if self.inputs.operation == "hdr_copy":
            path, base, _ = split_filename(value)
            _, base, ext = split_filename(self.inputs.operand_file)
            suffix = self.inputs.operation
            return os.path.join(path, "{0}{1}{2}".format(base, suffix, ext))
        else:
            return super(BinaryMaths, self)._overload_extension(value, name)


class BinaryMathsInputInteger(MathsInput):
    """Input Spec for seg_maths Binary operations that require integer."""

    operation = traits.Enum(
        "dil",
        "ero",
        "tp",
        "equal",
        "pad",
        "crop",
        mandatory=True,
        argstr="-%s",
        position=4,
        desc="""\
Operation to perform:

    * equal - <int> - Get voxels equal to <int>
    * dil - <int>  - Dilate the image <int> times (in voxels).
    * ero - <int> - Erode the image <int> times (in voxels).
    * tp - <int> - Extract time point <int>
    * crop - <int> - Crop <int> voxels around each 3D volume.
    * pad - <int> -  Pad <int> voxels with NaN value around each 3D volume.

""",
    )

    operand_value = traits.Int(
        argstr="%d",
        mandatory=True,
        position=5,
        desc="int value to perform operation with",
    )


class BinaryMathsInteger(MathsCommand):
    """Integer mathematical operations.

    See Also
    --------
    `Source code <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg>`__ --
    `Documentation <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg_documentation>`__

    Examples
    --------
    >>> import copy
    >>> from nipype.interfaces.niftyseg import BinaryMathsInteger
    >>> binaryi = BinaryMathsInteger()
    >>> binaryi.inputs.in_file = 'im1.nii'
    >>> binaryi.inputs.output_datatype = 'float'
    >>> # Test dil operation
    >>> binaryi_dil = copy.deepcopy(binaryi)
    >>> binaryi_dil.inputs.operation = 'dil'
    >>> binaryi_dil.inputs.operand_value = 2
    >>> binaryi_dil.cmdline
    'seg_maths im1.nii -dil 2 -odt float im1_dil.nii'
    >>> binaryi_dil.run()  # doctest: +SKIP
    >>> # Test dil operation
    >>> binaryi_ero = copy.deepcopy(binaryi)
    >>> binaryi_ero.inputs.operation = 'ero'
    >>> binaryi_ero.inputs.operand_value = 1
    >>> binaryi_ero.cmdline
    'seg_maths im1.nii -ero 1 -odt float im1_ero.nii'
    >>> binaryi_ero.run()  # doctest: +SKIP
    >>> # Test pad operation
    >>> binaryi_pad = copy.deepcopy(binaryi)
    >>> binaryi_pad.inputs.operation = 'pad'
    >>> binaryi_pad.inputs.operand_value = 4
    >>> binaryi_pad.cmdline
    'seg_maths im1.nii -pad 4 -odt float im1_pad.nii'
    >>> binaryi_pad.run()  # doctest: +SKIP

    """

    input_spec = BinaryMathsInputInteger


class TupleMathsInput(MathsInput):
    """Input Spec for seg_maths Tuple operations."""

    operation = traits.Enum(
        "lncc",
        "lssd",
        "lltsnorm",
        mandatory=True,
        argstr="-%s",
        position=4,
        desc="""\
Operation to perform:

    * lncc <file> <std> Local CC between current img and <file> on a kernel with <std>
    * lssd <file> <std> Local SSD between current img and <file> on a kernel with <std>
    * lltsnorm <file_norm> <float>  Linear LTS normalisation assuming <float> percent outliers

""",
    )

    operand_file1 = File(
        exists=True,
        argstr="%s",
        mandatory=True,
        position=5,
        xor=["operand_value1"],
        desc="image to perform operation 1 with",
    )

    desc = "float value to perform operation 1 with"
    operand_value1 = traits.Float(
        argstr="%.8f", mandatory=True, position=5, xor=["operand_file1"], desc=desc
    )

    operand_file2 = File(
        exists=True,
        argstr="%s",
        mandatory=True,
        position=6,
        xor=["operand_value2"],
        desc="image to perform operation 2 with",
    )

    desc = "float value to perform operation 2 with"
    operand_value2 = traits.Float(
        argstr="%.8f", mandatory=True, position=6, xor=["operand_file2"], desc=desc
    )


class TupleMaths(MathsCommand):
    """Mathematical operations on tuples.

    See Also
    --------
    `Source code <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg>`__ --
    `Documentation <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg_documentation>`__

    Examples
    --------
    >>> import copy
    >>> from nipype.interfaces import niftyseg
    >>> tuple = niftyseg.TupleMaths()
    >>> tuple.inputs.in_file = 'im1.nii'
    >>> tuple.inputs.output_datatype = 'float'

    >>> # Test lncc operation
    >>> tuple_lncc = copy.deepcopy(tuple)
    >>> tuple_lncc.inputs.operation = 'lncc'
    >>> tuple_lncc.inputs.operand_file1 = 'im2.nii'
    >>> tuple_lncc.inputs.operand_value2 = 2.0
    >>> tuple_lncc.cmdline
    'seg_maths im1.nii -lncc im2.nii 2.00000000 -odt float im1_lncc.nii'
    >>> tuple_lncc.run()  # doctest: +SKIP

    >>> # Test lssd operation
    >>> tuple_lssd = copy.deepcopy(tuple)
    >>> tuple_lssd.inputs.operation = 'lssd'
    >>> tuple_lssd.inputs.operand_file1 = 'im2.nii'
    >>> tuple_lssd.inputs.operand_value2 = 1.0
    >>> tuple_lssd.cmdline
    'seg_maths im1.nii -lssd im2.nii 1.00000000 -odt float im1_lssd.nii'
    >>> tuple_lssd.run()  # doctest: +SKIP

    >>> # Test lltsnorm operation
    >>> tuple_lltsnorm = copy.deepcopy(tuple)
    >>> tuple_lltsnorm.inputs.operation = 'lltsnorm'
    >>> tuple_lltsnorm.inputs.operand_file1 = 'im2.nii'
    >>> tuple_lltsnorm.inputs.operand_value2 = 0.01
    >>> tuple_lltsnorm.cmdline
    'seg_maths im1.nii -lltsnorm im2.nii 0.01000000 -odt float im1_lltsnorm.nii'
    >>> tuple_lltsnorm.run()  # doctest: +SKIP

    """

    input_spec = TupleMathsInput


class MergeInput(MathsInput):
    """Input Spec for seg_maths merge operation."""

    dimension = traits.Int(mandatory=True, desc="Dimension to merge the images.")
    merge_files = traits.List(
        File(exists=True),
        argstr="%s",
        mandatory=True,
        position=4,
        desc="List of images to merge to the working image <input>.",
    )


class Merge(MathsCommand):
    """Merge image files.

    See Also
    --------
    `Source code <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg>`__ --
    `Documentation <http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftySeg_documentation>`__

    Examples
    --------
    >>> from nipype.interfaces import niftyseg
    >>> node = niftyseg.Merge()
    >>> node.inputs.in_file = 'im1.nii'
    >>> files = ['im2.nii', 'im3.nii']
    >>> node.inputs.merge_files = files
    >>> node.inputs.dimension = 2
    >>> node.inputs.output_datatype = 'float'
    >>> node.cmdline
    'seg_maths im1.nii -merge 2 2 im2.nii im3.nii -odt float im1_merged.nii'

    """

    input_spec = MergeInput
    _suffix = "_merged"

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for seg_maths."""
        if opt == "merge_files":
            return "-merge %d %d %s" % (len(val), self.inputs.dimension, " ".join(val))

        return super(Merge, self)._format_arg(opt, spec, val)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..dti import ProbTrackX


def test_ProbTrackX_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        avoid_mp=dict(
            argstr="--avoid=%s",
            extensions=None,
        ),
        c_thresh=dict(
            argstr="--cthr=%.3f",
        ),
        correct_path_distribution=dict(
            argstr="--pd",
        ),
        dist_thresh=dict(
            argstr="--distthresh=%.3f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fibst=dict(
            argstr="--fibst=%d",
        ),
        force_dir=dict(
            argstr="--forcedir",
            usedefault=True,
        ),
        fsamples=dict(
            mandatory=True,
        ),
        inv_xfm=dict(
            argstr="--invxfm=%s",
            extensions=None,
        ),
        loop_check=dict(
            argstr="--loopcheck",
        ),
        mask=dict(
            argstr="-m %s",
            extensions=None,
            mandatory=True,
        ),
        mask2=dict(
            argstr="--mask2=%s",
            extensions=None,
        ),
        mesh=dict(
            argstr="--mesh=%s",
            extensions=None,
        ),
        mod_euler=dict(
            argstr="--modeuler",
        ),
        mode=dict(
            argstr="--mode=%s",
            genfile=True,
        ),
        n_samples=dict(
            argstr="--nsamples=%d",
            usedefault=True,
        ),
        n_steps=dict(
            argstr="--nsteps=%d",
        ),
        network=dict(
            argstr="--network",
        ),
        opd=dict(
            argstr="--opd",
            usedefault=True,
        ),
        os2t=dict(
            argstr="--os2t",
        ),
        out_dir=dict(
            argstr="--dir=%s",
            genfile=True,
        ),
        output_type=dict(),
        phsamples=dict(
            mandatory=True,
        ),
        rand_fib=dict(
            argstr="--randfib=%d",
        ),
        random_seed=dict(
            argstr="--rseed",
        ),
        s2tastext=dict(
            argstr="--s2tastext",
        ),
        sample_random_points=dict(
            argstr="--sampvox",
        ),
        samples_base_name=dict(
            argstr="--samples=%s",
            usedefault=True,
        ),
        seed=dict(
            argstr="--seed=%s",
            mandatory=True,
        ),
        seed_ref=dict(
            argstr="--seedref=%s",
            extensions=None,
        ),
        step_length=dict(
            argstr="--steplength=%.3f",
        ),
        stop_mask=dict(
            argstr="--stop=%s",
            extensions=None,
        ),
        target_masks=dict(
            argstr="--targetmasks=%s",
        ),
        thsamples=dict(
            mandatory=True,
        ),
        use_anisotropy=dict(
            argstr="--usef",
        ),
        verbose=dict(
            argstr="--verbose=%d",
        ),
        waypoints=dict(
            argstr="--waypoints=%s",
            extensions=None,
        ),
        xfm=dict(
            argstr="--xfm=%s",
            extensions=None,
        ),
    )
    inputs = ProbTrackX.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ProbTrackX_outputs():
    output_map = dict(
        fdt_paths=dict(),
        log=dict(
            extensions=None,
        ),
        particle_files=dict(),
        targets=dict(),
        way_total=dict(
            extensions=None,
        ),
    )
    outputs = ProbTrackX.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..segmentation import Atropos


def test_Atropos_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        convergence_threshold=dict(
            requires=["n_iterations"],
        ),
        dimension=dict(
            argstr="--image-dimensionality %d",
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        icm_use_synchronous_update=dict(
            argstr="%s",
        ),
        initialization=dict(
            argstr="%s",
            mandatory=True,
            requires=["number_of_tissue_classes"],
        ),
        intensity_images=dict(
            argstr="--intensity-image %s...",
            mandatory=True,
        ),
        kmeans_init_centers=dict(),
        likelihood_model=dict(
            argstr="--likelihood-model %s",
        ),
        mask_image=dict(
            argstr="--mask-image %s",
            extensions=None,
            mandatory=True,
        ),
        maximum_number_of_icm_terations=dict(
            requires=["icm_use_synchronous_update"],
        ),
        mrf_radius=dict(
            requires=["mrf_smoothing_factor"],
        ),
        mrf_smoothing_factor=dict(
            argstr="%s",
        ),
        n_iterations=dict(
            argstr="%s",
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        number_of_tissue_classes=dict(
            mandatory=True,
        ),
        out_classified_image_name=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_posteriors_name_template=dict(
            usedefault=True,
        ),
        posterior_formulation=dict(
            argstr="%s",
        ),
        prior_image=dict(),
        prior_probability_threshold=dict(
            requires=["prior_weighting"],
        ),
        prior_weighting=dict(),
        save_posteriors=dict(),
        use_mixture_model_proportions=dict(
            requires=["posterior_formulation"],
        ),
        use_random_seed=dict(
            argstr="--use-random-seed %d",
            usedefault=True,
        ),
    )
    inputs = Atropos.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Atropos_outputs():
    output_map = dict(
        classified_image=dict(
            extensions=None,
        ),
        posteriors=dict(),
    )
    outputs = Atropos.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import os
import numpy as np

from ...utils.filemanip import (
    split_filename,
    fname_presuffix,
    ensure_list,
    simplify_list,
)
from ..base import TraitedSpec, isdefined, File, traits, OutputMultiPath, InputMultiPath
from .base import SPMCommandInputSpec, SPMCommand, scans_for_fnames, scans_for_fname


class Analyze2niiInputSpec(SPMCommandInputSpec):
    analyze_file = File(exists=True, mandatory=True)


class Analyze2niiOutputSpec(SPMCommandInputSpec):
    nifti_file = File(exists=True)


class Analyze2nii(SPMCommand):

    input_spec = Analyze2niiInputSpec
    output_spec = Analyze2niiOutputSpec

    def _make_matlab_command(self, _):
        script = "V = spm_vol('%s');\n" % self.inputs.analyze_file
        _, name, _ = split_filename(self.inputs.analyze_file)
        self.output_name = os.path.join(os.getcwd(), name + ".nii")
        script += "[Y, XYZ] = spm_read_vols(V);\n"
        script += "V.fname = '%s';\n" % self.output_name
        script += "spm_write_vol(V, Y);\n"

        return script

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["nifti_file"] = self.output_name
        return outputs


class CalcCoregAffineInputSpec(SPMCommandInputSpec):
    target = File(
        exists=True, mandatory=True, desc="target for generating affine transform"
    )
    moving = File(
        exists=True,
        mandatory=True,
        copyfile=False,
        desc=("volume transform can be applied to register with " "target"),
    )
    mat = File(desc="Filename used to store affine matrix")
    invmat = File(desc="Filename used to store inverse affine matrix")


class CalcCoregAffineOutputSpec(TraitedSpec):
    mat = File(exists=True, desc="Matlab file holding transform")
    invmat = File(desc="Matlab file holding inverse transform")


class CalcCoregAffine(SPMCommand):
    """Uses SPM (spm_coreg) to calculate the transform mapping
    moving to target. Saves Transform in mat (matlab binary file)
    Also saves inverse transform

    Examples
    --------

    >>> import nipype.interfaces.spm.utils as spmu
    >>> coreg = spmu.CalcCoregAffine(matlab_cmd='matlab-spm8')
    >>> coreg.inputs.target = 'structural.nii'
    >>> coreg.inputs.moving = 'functional.nii'
    >>> coreg.inputs.mat = 'func_to_struct.mat'
    >>> coreg.run() # doctest: +SKIP

    .. note::

     * the output file mat is saves as a matlab binary file
     * calculating the transforms does NOT change either input image
       it does not **move** the moving image, only calculates the transform
       that can be used to move it
    """

    input_spec = CalcCoregAffineInputSpec
    output_spec = CalcCoregAffineOutputSpec

    def _make_inv_file(self):
        """makes filename to hold inverse transform if not specified"""
        invmat = fname_presuffix(self.inputs.mat, prefix="inverse_")
        return invmat

    def _make_mat_file(self):
        """makes name for matfile if doesn exist"""
        pth, mv, _ = split_filename(self.inputs.moving)
        _, tgt, _ = split_filename(self.inputs.target)
        mat = os.path.join(pth, "%s_to_%s.mat" % (mv, tgt))
        return mat

    def _make_matlab_command(self, _):
        """checks for SPM, generates script"""
        if not isdefined(self.inputs.mat):
            self.inputs.mat = self._make_mat_file()
        if not isdefined(self.inputs.invmat):
            self.inputs.invmat = self._make_inv_file()
        script = """
        target = '%s';
        moving = '%s';
        targetv = spm_vol(target);
        movingv = spm_vol(moving);
        x = spm_coreg(targetv, movingv);
        M = spm_matrix(x);
        save('%s' , 'M' );
        M = inv(M);
        save('%s','M')
        """ % (
            self.inputs.target,
            self.inputs.moving,
            self.inputs.mat,
            self.inputs.invmat,
        )
        return script

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["mat"] = os.path.abspath(self.inputs.mat)
        outputs["invmat"] = os.path.abspath(self.inputs.invmat)
        return outputs


class ApplyTransformInputSpec(SPMCommandInputSpec):
    in_file = File(
        exists=True,
        mandatory=True,
        copyfile=True,
        desc="file to apply transform to, (only updates header)",
    )
    mat = File(exists=True, mandatory=True, desc="file holding transform to apply")
    out_file = File(desc="output file name for transformed data", genfile=True)


class ApplyTransformOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc="Transformed image file")


class ApplyTransform(SPMCommand):
    """Uses SPM to apply transform stored in a .mat file to given file

    Examples
    --------

    >>> import nipype.interfaces.spm.utils as spmu
    >>> applymat = spmu.ApplyTransform()
    >>> applymat.inputs.in_file = 'functional.nii'
    >>> applymat.inputs.mat = 'func_to_struct.mat'
    >>> applymat.run() # doctest: +SKIP

    """

    input_spec = ApplyTransformInputSpec
    output_spec = ApplyTransformOutputSpec

    def _make_matlab_command(self, _):
        """checks for SPM, generates script"""
        outputs = self._list_outputs()
        self.inputs.out_file = outputs["out_file"]
        script = """
        infile = '%s';
        outfile = '%s'
        transform = load('%s');

        V = spm_vol(infile);
        X = spm_read_vols(V);
        [p n e v] = spm_fileparts(V.fname);
        V.mat = transform.M * V.mat;
        V.fname = fullfile(outfile);
        spm_write_vol(V,X);

        """ % (
            self.inputs.in_file,
            self.inputs.out_file,
            self.inputs.mat,
        )
        # img_space = spm_get_space(infile);
        # spm_get_space(infile, transform.M * img_space);
        return script

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if not isdefined(self.inputs.out_file):
            outputs["out_file"] = os.path.abspath(self._gen_outfilename())
        else:
            outputs["out_file"] = os.path.abspath(self.inputs.out_file)
        return outputs

    def _gen_outfilename(self):
        _, name, _ = split_filename(self.inputs.in_file)
        return name + "_trans.nii"


class ResliceInputSpec(SPMCommandInputSpec):
    in_file = File(
        exists=True,
        mandatory=True,
        desc="file to apply transform to, (only updates header)",
    )
    space_defining = File(
        exists=True, mandatory=True, desc="Volume defining space to slice in_file into"
    )

    interp = traits.Range(
        low=0,
        high=7,
        usedefault=True,
        desc="degree of b-spline used for interpolation"
        "0 is nearest neighbor (default)",
    )

    out_file = File(desc="Optional file to save resliced volume")


class ResliceOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc="resliced volume")


class Reslice(SPMCommand):
    """uses  spm_reslice to resample in_file into space of space_defining"""

    input_spec = ResliceInputSpec
    output_spec = ResliceOutputSpec

    def _make_matlab_command(self, _):
        """generates script"""
        if not isdefined(self.inputs.out_file):
            self.inputs.out_file = fname_presuffix(self.inputs.in_file, prefix="r")
        script = """
        flags.mean = 0;
        flags.which = 1;
        flags.mask = 0;
        flags.interp = %d;
        infiles = strvcat(\'%s\', \'%s\');
        invols = spm_vol(infiles);
        spm_reslice(invols, flags);
        """ % (
            self.inputs.interp,
            self.inputs.space_defining,
            self.inputs.in_file,
        )
        return script

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_file"] = os.path.abspath(self.inputs.out_file)
        return outputs


class ApplyInverseDeformationInput(SPMCommandInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        mandatory=True,
        field="fnames",
        desc="Files on which deformation is applied",
    )
    target = File(
        exists=True, field="comp{1}.inv.space", desc="File defining target space"
    )
    deformation = File(
        exists=True,
        field="comp{1}.inv.comp{1}.sn2def.matname",
        desc="SN SPM deformation file",
        xor=["deformation_field"],
    )
    deformation_field = File(
        exists=True,
        field="comp{1}.inv.comp{1}.def",
        desc="SN SPM deformation file",
        xor=["deformation"],
    )
    interpolation = traits.Range(
        low=0, high=7, field="interp", desc="degree of b-spline used for interpolation"
    )

    bounding_box = traits.List(
        traits.Float(),
        field="comp{1}.inv.comp{1}.sn2def.bb",
        minlen=6,
        maxlen=6,
        desc="6-element list (opt)",
    )
    voxel_sizes = traits.List(
        traits.Float(),
        field="comp{1}.inv.comp{1}.sn2def.vox",
        minlen=3,
        maxlen=3,
        desc="3-element list (opt)",
    )


class ApplyInverseDeformationOutput(TraitedSpec):
    out_files = OutputMultiPath(File(exists=True), desc="Transformed files")


class ApplyInverseDeformation(SPMCommand):
    """Uses spm to apply inverse deformation stored in a .mat file or a
    deformation field to a given file

    Examples
    --------

    >>> import nipype.interfaces.spm.utils as spmu
    >>> inv = spmu.ApplyInverseDeformation()
    >>> inv.inputs.in_files = 'functional.nii'
    >>> inv.inputs.deformation = 'struct_to_func.mat'
    >>> inv.inputs.target = 'structural.nii'
    >>> inv.run() # doctest: +SKIP
    """

    input_spec = ApplyInverseDeformationInput
    output_spec = ApplyInverseDeformationOutput

    _jobtype = "util"
    _jobname = "defs"

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm"""
        if opt == "in_files":
            return scans_for_fnames(ensure_list(val))
        if opt == "target":
            return scans_for_fname(ensure_list(val))
        if opt == "deformation":
            return np.array([simplify_list(val)], dtype=object)
        if opt == "deformation_field":
            return np.array([simplify_list(val)], dtype=object)
        return val

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_files"] = []
        for filename in self.inputs.in_files:
            _, fname = os.path.split(filename)
            outputs["out_files"].append(os.path.realpath("w%s" % fname))
        return outputs


class ResliceToReferenceInput(SPMCommandInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        mandatory=True,
        field="fnames",
        desc="Files on which deformation is applied",
    )
    target = File(
        exists=True, field="comp{1}.id.space", desc="File defining target space"
    )
    interpolation = traits.Range(
        low=0, high=7, field="interp", desc="degree of b-spline used for interpolation"
    )

    bounding_box = traits.List(
        traits.Float(),
        field="comp{2}.idbbvox.bb",
        minlen=6,
        maxlen=6,
        desc="6-element list (opt)",
    )
    voxel_sizes = traits.List(
        traits.Float(),
        field="comp{2}.idbbvox.vox",
        minlen=3,
        maxlen=3,
        desc="3-element list (opt)",
    )


class ResliceToReferenceOutput(TraitedSpec):
    out_files = OutputMultiPath(File(exists=True), desc="Transformed files")


class ResliceToReference(SPMCommand):
    """Uses spm to reslice a volume to a target image space or to a provided
    voxel size and bounding box

    Examples
    --------

    >>> import nipype.interfaces.spm.utils as spmu
    >>> r2ref = spmu.ResliceToReference()
    >>> r2ref.inputs.in_files = 'functional.nii'
    >>> r2ref.inputs.target = 'structural.nii'
    >>> r2ref.run() # doctest: +SKIP
    """

    input_spec = ResliceToReferenceInput
    output_spec = ResliceToReferenceOutput

    _jobtype = "util"
    _jobname = "defs"

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm"""
        if opt == "in_files":
            return scans_for_fnames(ensure_list(val))
        if opt == "target":
            return scans_for_fname(ensure_list(val))
        if opt == "deformation":
            return np.array([simplify_list(val)], dtype=object)
        if opt == "deformation_field":
            return np.array([simplify_list(val)], dtype=object)
        return val

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_files"] = []
        for filename in self.inputs.in_files:
            _, fname = os.path.split(filename)
            outputs["out_files"].append(os.path.realpath("w%s" % fname))
        return outputs


class DicomImportInputSpec(SPMCommandInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        mandatory=True,
        field="data",
        desc="dicom files to be converted",
    )
    output_dir_struct = traits.Enum(
        "flat",
        "series",
        "patname",
        "patid_date",
        "patid",
        "date_time",
        field="root",
        usedefault=True,
        desc="directory structure for the output.",
    )
    output_dir = traits.Str(
        "./converted_dicom", field="outdir", usedefault=True, desc="output directory."
    )
    format = traits.Enum(
        "nii", "img", field="convopts.format", usedefault=True, desc="output format."
    )
    icedims = traits.Bool(
        False,
        field="convopts.icedims",
        usedefault=True,
        desc=(
            "If image sorting fails, one can try using "
            "the additional SIEMENS ICEDims information "
            "to create unique filenames. Use this only if "
            "there would be multiple volumes with exactly "
            "the same file names."
        ),
    )


class DicomImportOutputSpec(TraitedSpec):
    out_files = OutputMultiPath(File(exists=True), desc="converted files")


class DicomImport(SPMCommand):
    """Uses spm to convert DICOM files to nii or img+hdr.

    Examples
    --------

    >>> import nipype.interfaces.spm.utils as spmu
    >>> di = spmu.DicomImport()
    >>> di.inputs.in_files = ['functional_1.dcm', 'functional_2.dcm']
    >>> di.run() # doctest: +SKIP
    """

    input_spec = DicomImportInputSpec
    output_spec = DicomImportOutputSpec

    _jobtype = "util"
    _jobname = "dicom"

    def _format_arg(self, opt, spec, val):
        """Convert input to appropriate format for spm"""
        if opt == "in_files":
            return np.array(val, dtype=object)
        if opt == "output_dir":
            return np.array([val], dtype=object)
        if opt == "output_dir":
            return os.path.abspath(val)
        if opt == "icedims":
            if val:
                return 1
            return 0
        return super(DicomImport, self)._format_arg(opt, spec, val)

    def _run_interface(self, runtime):
        od = os.path.abspath(self.inputs.output_dir)
        if not os.path.isdir(od):
            os.mkdir(od)
        return super(DicomImport, self)._run_interface(runtime)

    def _list_outputs(self):
        from glob import glob

        outputs = self._outputs().get()
        od = os.path.abspath(self.inputs.output_dir)

        ext = self.inputs.format
        if self.inputs.output_dir_struct == "flat":
            outputs["out_files"] = glob(os.path.join(od, "*.%s" % ext))
        elif self.inputs.output_dir_struct == "series":
            outputs["out_files"] = glob(
                os.path.join(od, os.path.join("*", "*.%s" % ext))
            )
        elif self.inputs.output_dir_struct in ["patid", "date_time", "patname"]:
            outputs["out_files"] = glob(
                os.path.join(od, os.path.join("*", "*", "*.%s" % ext))
            )
        elif self.inputs.output_dir_struct == "patid_date":
            outputs["out_files"] = glob(
                os.path.join(od, os.path.join("*", "*", "*", "*.%s" % ext))
            )
        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import MakeSurfaces


def test_MakeSurfaces_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        copy_inputs=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fix_mtl=dict(
            argstr="-fix_mtl",
        ),
        hemisphere=dict(
            argstr="%s",
            mandatory=True,
            position=-1,
        ),
        in_T1=dict(
            argstr="-T1 %s",
            extensions=None,
        ),
        in_aseg=dict(
            argstr="-aseg %s",
            extensions=None,
        ),
        in_filled=dict(
            extensions=None,
            mandatory=True,
        ),
        in_label=dict(
            extensions=None,
            xor=["noaparc"],
        ),
        in_orig=dict(
            argstr="-orig %s",
            extensions=None,
            mandatory=True,
        ),
        in_white=dict(
            extensions=None,
        ),
        in_wm=dict(
            extensions=None,
            mandatory=True,
        ),
        longitudinal=dict(
            argstr="-long",
        ),
        maximum=dict(
            argstr="-max %.1f",
        ),
        mgz=dict(
            argstr="-mgz",
        ),
        no_white=dict(
            argstr="-nowhite",
        ),
        noaparc=dict(
            argstr="-noaparc",
            xor=["in_label"],
        ),
        orig_pial=dict(
            argstr="-orig_pial %s",
            extensions=None,
            requires=["in_label"],
        ),
        orig_white=dict(
            argstr="-orig_white %s",
            extensions=None,
        ),
        subject_id=dict(
            argstr="%s",
            mandatory=True,
            position=-2,
            usedefault=True,
        ),
        subjects_dir=dict(),
        white=dict(
            argstr="-white %s",
        ),
        white_only=dict(
            argstr="-whiteonly",
        ),
    )
    inputs = MakeSurfaces.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MakeSurfaces_outputs():
    output_map = dict(
        out_area=dict(
            extensions=None,
        ),
        out_cortex=dict(
            extensions=None,
        ),
        out_curv=dict(
            extensions=None,
        ),
        out_pial=dict(
            extensions=None,
        ),
        out_thickness=dict(
            extensions=None,
        ),
        out_white=dict(
            extensions=None,
        ),
    )
    outputs = MakeSurfaces.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..tensors import EstimateResponseForSH


def test_EstimateResponseForSH_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        debug=dict(
            argstr="-debug",
        ),
        encoding_file=dict(
            argstr="-grad %s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        mask_image=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        maximum_harmonic_order=dict(
            argstr="-lmax %s",
        ),
        normalise=dict(
            argstr="-normalise",
        ),
        out_filename=dict(
            argstr="%s",
            extensions=None,
            genfile=True,
            position=-1,
        ),
        quiet=dict(
            argstr="-quiet",
        ),
    )
    inputs = EstimateResponseForSH.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_EstimateResponseForSH_outputs():
    output_map = dict(
        response=dict(
            extensions=None,
        ),
    )
    outputs = EstimateResponseForSH.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..epi import ApplyTOPUP


def test_ApplyTOPUP_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        datatype=dict(
            argstr="-d=%s",
        ),
        encoding_file=dict(
            argstr="--datain=%s",
            extensions=None,
            mandatory=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_files=dict(
            argstr="--imain=%s",
            mandatory=True,
            sep=",",
        ),
        in_index=dict(
            argstr="--inindex=%s",
            sep=",",
        ),
        in_topup_fieldcoef=dict(
            argstr="--topup=%s",
            copyfile=False,
            extensions=None,
            requires=["in_topup_movpar"],
        ),
        in_topup_movpar=dict(
            copyfile=False,
            extensions=None,
            requires=["in_topup_fieldcoef"],
        ),
        interp=dict(
            argstr="--interp=%s",
        ),
        method=dict(
            argstr="--method=%s",
        ),
        out_corrected=dict(
            argstr="--out=%s",
            extensions=None,
            name_source=["in_files"],
            name_template="%s_corrected",
        ),
        output_type=dict(),
    )
    inputs = ApplyTOPUP.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ApplyTOPUP_outputs():
    output_map = dict(
        out_corrected=dict(
            extensions=None,
        ),
    )
    outputs = ApplyTOPUP.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
